/****************************************************************************
 *
 * PROGRAM:      clump4p
 *
 * AUTHOR(S):    Pawel Netzel - SIL UC
 *               Jarek Jasiewicz - SIL UC
 *                       (the clumping algorithm for a region)
 *
 * PURPOSE:      Recategorizes data in a raster map layer by grouping cells
 *               that form physically discrete areas into unique categories.
 *
 * COPYRIGHT:    (C) 2012 by Pawel Netzel
 *
 *               This program is a free software under the GNU General Public
 *               License (>=v2).
 *
 ***************************************************************************/

#ifndef _DATA_H_
#define _DATA_H_

#include <ezgdal.h>
#include "main.h"

#define REGION_SIZE 500
#define INCR 4096

typedef struct {
	int rowMin,rowMax;
	int colMin,colMax;
	int nLabels;  // max labal number
	int *labels; // array from 1
	int regionUp;
} Region;

extern int totalIndex;

extern int nRegions;
extern int nRegionCols;
extern int nRegionRows;
extern Region *regions;

extern int nDataCols;
extern int nDataRows;

extern int *borderDataBuffer, *borderResultBuffer;
extern int **dataBuffer;
extern int **resultBuffer;
extern int **calcBuffer;

void initVariables(EZGDAL_LAYER *);
void freeVariables();

void loadDataBuffer(EZGDAL_LAYER *, int, int);
void saveResultBuffer(EZGDAL_LAYER *, int, int);

void dumpArray(int *, int);
void dumpRegions(char *);

#endif

